#include "w5100.h"
#include "spi_functions.h"
#include <alt_types.h>
#include <stdio.h>

#define WRITE_OPCODE      0xF0
#define READ_OPCODE       0x0F

unsigned char _ip_[4];
unsigned short _port_;

static unsigned char I_STATUS[MAX_SOCK_NUM];
static unsigned short SMASK[MAX_SOCK_NUM]; ///< Variable for Tx buffer MASK in each channel 
static unsigned short RMASK[MAX_SOCK_NUM]; ///< Variable for Rx buffer MASK in each channel 
static unsigned short SSIZE[MAX_SOCK_NUM]; ///< Max Tx buffer size by each channel 
static unsigned short RSIZE[MAX_SOCK_NUM]; ///< Max Rx buffer size by each channel 
static unsigned short SBUFBASEADDRESS[MAX_SOCK_NUM]; ///< Tx buffer base address by each channel 
static unsigned short RBUFBASEADDRESS[MAX_SOCK_NUM]; ///< Rx buffer base address by each channel 

void setIP(unsigned char ip1, unsigned char ip2, unsigned char ip3,
		unsigned char ip4) {
	_ip_[0] = ip1;
	_ip_[1] = ip2;
	_ip_[2] = ip3;
	_ip_[3] = ip4;
}

void getIP(unsigned char *ip1, unsigned char *ip2, unsigned char *ip3,
		unsigned char *ip4) {
	*ip1 = _ip_[0];
	*ip2 = _ip_[1];
	*ip3 = _ip_[2];
	*ip4 = _ip_[3];
}

void set_port(unsigned short puerto_fuente) {
	_port_ = puerto_fuente;
}

void get_port(unsigned short *puerto_fuente) {
	*puerto_fuente = _port_;
}

unsigned char getISR(unsigned char s) {
	return I_STATUS[s];
}

void putISR(unsigned char s, unsigned char val) {
	I_STATUS[s] = val;
}

unsigned short getIINCHIP_RxMAX(unsigned char s) {
	return RSIZE[s];
}
unsigned short getIINCHIP_TxMAX(unsigned char s) {
	return SSIZE[s];
}
unsigned short getIINCHIP_RxMASK(unsigned char s) {
	return RMASK[s];
}
unsigned short getIINCHIP_TxMASK(unsigned char s) {
	return SMASK[s];
}
unsigned short getIINCHIP_RxBASE(unsigned char s) {
	return RBUFBASEADDRESS[s];
}
unsigned short getIINCHIP_TxBASE(unsigned char s) {
	return SBUFBASEADDRESS[s];
}

bool write_wiznet(unsigned short address, unsigned char data) {

	if (spi_write(WIZNET_WRITE, address, data) != 0x03) {

#if DEBUG_W5100>0
		printf("Falla en la comunicacion SPI \n");
#endif

		return false;
	}

	return true;
}

unsigned char read_wiznet(unsigned short address) {

	unsigned char temp_data;

	temp_data = spi_write(WIZNET_READ, address, 0x00);

	return temp_data;
}

void iinchip_init(void) {
	wr_MR( RST);
}

/**
 brief	This function set the transmit & receive buffer size as per the channels is used
 Note for TMSR and RMSR bits are as follows\n
 bit 1-0 : memory size of channel #0 \n
 bit 3-2 : memory size of channel #1 \n
 bit 5-4 : memory size of channel #2 \n
 bit 7-6 : memory size of channel #3 \n\n
 Maximum memory size for Tx, Rx in the W5100 is 8K Bytes,\n
 In the range of 8KBytes, the memory size could be allocated dynamically by each channel.\n
 Be attentive to sum of memory size shouldn't exceed 8Kbytes\n
 and to data transmission and receiption from non-allocated channel may cause some problems.\n
 If the 8KBytes memory is already  assigned to centain channel, \n
 other 3 channels couldn't be used, for there's no available memory.\n
 If two 4KBytes memory are assigned to two each channels, \n
 other 2 channels couldn't be used, for there's no available memory.\n
 */

void sysinit(unsigned char tx_size, ///< tx_size Tx memory size (00 - 1KByte, 01- 2KBtye, 10 - 4KByte, 11 - 8KByte)
		unsigned char rx_size ///< rx_size Rx memory size (00 - 1KByte, 01- 2KBtye, 10 - 4KByte, 11 - 8KByte)
		) {
	short i;
	short ssum, rsum;
#if DEBUG_W5100>0
	printf("sysinit()\r\n");
#endif
	ssum = 0;
	rsum = 0;

	write_wiznet(TMSR, tx_size);  //// Set Tx memory size for each channel
	write_wiznet(RMSR, rx_size);	//// Set Rx memory size for each channel
#if DEBUG_W5100>0
			printf("Channel : SEND MEM SIZE : RECV MEM SIZE\r\n");
#endif

	for (i = 0; i < 4; i++) // Set the size, masking and base address of Tx & Rx memory by each channel
			{
		SSIZE[i] = (short) (0);
		RSIZE[i] = (short) (0);
		if (ssum < 8192) {
			switch ((tx_size >> i * 2) & 0x03)  // Set Tx memory size
			{
			case 0:
				SSIZE[i] = (short) (1024);
				SMASK[i] = (unsigned short) (0x03FF);
				break;
			case 1:
				SSIZE[i] = (short) (2048);
				SMASK[i] = (unsigned short) (0x07FF);
				break;
			case 2:
				SSIZE[i] = (short) (4096);
				SMASK[i] = (unsigned short) (0x0FFF);
				break;
			case 3:
				SSIZE[i] = (short) (8192);
				SMASK[i] = (unsigned short) (0x1FFF);
				break;
			}
		}
		if (rsum < 8192) {
			switch ((rx_size >> i * 2) & 0x03)     // Set Rx memory size
			{
			case 0:
				RSIZE[i] = (short) (1024);
				RMASK[i] = (unsigned short) (0x03FF);
				break;
			case 1:
				RSIZE[i] = (short) (2048);
				RMASK[i] = (unsigned short) (0x07FF);
				break;
			case 2:
				RSIZE[i] = (short) (4096);
				RMASK[i] = (unsigned short) (0x0FFF);
				break;
			case 3:
				RSIZE[i] = (short) (8192);
				RMASK[i] = (unsigned short) (0x1FFF);
				break;
			}
		}
		ssum += SSIZE[i];
		rsum += RSIZE[i];

//     if (i != 0)             // Sets base address of Tx and Rx memory for channel #1,#2,#3
//		{
//			SBUFBASEADDRESS[i] = SBUFBASEADDRESS[i-1] + SSIZE[i-1];
//			RBUFBASEADDRESS[i] = RBUFBASEADDRESS[i-1] + RSIZE[i-1];
//		}

#if DEBUG_W5100>0
		printf("%d : %.4x : %.4x : %.4x : %.4x\r\n", i, (unsigned short)SBUFBASEADDRESS[i], (unsigned short)RBUFBASEADDRESS[i], SSIZE[i], RSIZE[i]);
#endif
	}
}

///// Rutina de reset del W5100
void reset_w5100(void) {
	reset_wiznet();
	//write_wiznet(MR, RST);
}

//// escritura directa en el registro MR
void wr_MR(unsigned char val) {

	write_wiznet(MR, val);
	reset_w5100();
}

////funcion solo para leer el reg MR en debugging
void rd_MR(void) {
	/*acceso directo al reg MR */
#if DEBUG_W5100>0
	//printf(" leyendo reg MR %d \r\n", read_wiznet(MR));
#endif
}

/**
 esta fuincion envia la gateway IP address(GAR) al w5100.
 */
void wr_GAR(unsigned char * addr) { /**< addr puntero  a un 4 -byte array responsible to set the Gateway IP address. */
	write_wiznet((GAR + 0), addr[0]);
	write_wiznet((GAR + 1), addr[1]);
	write_wiznet((GAR + 2), addr[2]);
	write_wiznet((GAR + 3), addr[3]);
}

/// LEE direccin de puerta de enlace  y la alamacena en un vector
void rd_GAR(unsigned char * addr) {
	addr[0] = read_wiznet((GAR + 0));
	addr[1] = read_wiznet((GAR + 1));
	addr[2] = read_wiznet((GAR + 2));
	addr[3] = read_wiznet((GAR + 3));
}

/**
 funcion que configura la mascara de subred(SubnetMask SUB)*/
void wr_SUBR(unsigned char * addr) {
	write_wiznet((SUBR + 0), addr[0]);
	write_wiznet((SUBR + 1), addr[1]);
	write_wiznet((SUBR + 2), addr[2]);
	write_wiznet((SUBR + 3), addr[3]);

}

/// LEE mascara de subred  y la almacena en un vector
void rd_SUBR(unsigned char * addr) {
	addr[0] = read_wiznet((SUBR + 0));
	addr[1] = read_wiznet((SUBR + 1));
	addr[2] = read_wiznet((SUBR + 2));
	addr[3] = read_wiznet((SUBR + 3));
}

/**
 Funcin que configura el numero de maquina (MAC)
 Source Hardware Address (SHAR)
 */
void wr_SHAR(unsigned char * addr) { /** addr puntero a vector de 6 -bytes**/
	write_wiznet((SHAR + 0), addr[0]);
	write_wiznet((SHAR + 1), addr[1]);
	write_wiznet((SHAR + 2), addr[2]);
	write_wiznet((SHAR + 3), addr[3]);
	write_wiznet((SHAR + 4), addr[4]);
	write_wiznet((SHAR + 5), addr[5]);
}

/// LEE direccin de puerta de enlace a la almacena en un vector
void rd_SHAR(unsigned char * addr) {
	addr[0] = read_wiznet((SHAR + 0));
	addr[1] = read_wiznet((SHAR + 1));
	addr[2] = read_wiznet((SHAR + 2));
	addr[3] = read_wiznet((SHAR + 3));
	addr[4] = read_wiznet((SHAR + 4));
	addr[5] = read_wiznet((SHAR + 5));
}

/**
 funcin para configuracion de direccin ip fuente
 la ip que tendra el w5100
 */
void wr_SIPR(unsigned char * addr) {
	write_wiznet((SIPR + 0), addr[0]);
	write_wiznet((SIPR + 1), addr[1]);
	write_wiznet((SIPR + 2), addr[2]);
	write_wiznet((SIPR + 3), addr[3]);

}

/// LEE direccin IP del W5100 y la almacena en un vector
void rd_SIPR(unsigned char * addr) {
	addr[0] = read_wiznet((SIPR + 0));
	addr[1] = read_wiznet((SIPR + 1));
	addr[2] = read_wiznet((SIPR + 2));
	addr[3] = read_wiznet((SIPR + 3));

}

/**
 Esta funcion lee el registro de interrupciones in common register.
 */
unsigned char rd_IR(void) {
	return read_wiznet(IR);

}

/**
 Esta funcion configura el tiempo de retransmicion (timeout)
 el w5100 espera un tiempo(timeout)  hasta que el servidor le responda
 sino vuelve a retransmitir el paquete
 RTR (Retry Time-value Register)***/
void wr_RTR(unsigned short timeout) {
	write_wiznet(RTR, (unsigned char) ((timeout & 0xff00) >> 8));
	write_wiznet((RTR + 1), (unsigned char) (timeout & 0x00ff));
}

/**
 funcion que configura el numero de retransmicin

 If there is no response from the peer or delay in response then recorded time
 as per RTR & RCR register seeting then time out will occur.
 */
void wr_RCR(unsigned char retry) {
	write_wiznet(RCR, retry);
}

/**
 funcin que configura la mnascara de interrupciones
 Enable/Disable appropriate Interrupt. ('1' : interrupt enable)
 si algun bit en IMR es "0" no hay sealde interrupcion.
 */
void wr_IMR(unsigned char mask) {
	write_wiznet(IMR, mask); // must be setted 0x10.
}

/////// configuracion de parametros del destino o servidor al cual se l
//////enviaran los datos

void wr_Sn_DHAR(SOCKET s, unsigned char * addr) {
	write_wiznet((Sn_DHAR(s) + 0), addr[0]);
	write_wiznet((Sn_DHAR(s) + 1), addr[1]);
	write_wiznet((Sn_DHAR(s) + 2), addr[2]);
	write_wiznet((Sn_DHAR(s) + 3), addr[3]);
	write_wiznet((Sn_DHAR(s) + 4), addr[4]);
	write_wiznet((Sn_DHAR(s) + 5), addr[5]);
}
void rd_Sn_DHAR(SOCKET s, unsigned char * addr) {
	addr[0] = read_wiznet(Sn_DHAR(s));
	addr[1] = read_wiznet(Sn_DHAR(s) + 1);
	addr[2] = read_wiznet(Sn_DHAR(s) + 2);
	addr[3] = read_wiznet(Sn_DHAR(s) + 3);
	addr[4] = read_wiznet(Sn_DHAR(s) + 4);
	addr[5] = read_wiznet(Sn_DHAR(s) + 5);

	/*printf("leyendo numero MAC destino: %.2x:%.2x:%.2x:%.2x:%.2x:%.2x\n",\
read_wiznet(Sn_DHAR(s) + 0),read_wiznet(Sn_DHAR(s) + 1),read_wiznet(Sn_DHAR(s) + 2),read_wiznet(Sn_DHAR(s) + 3),\
read_wiznet(Sn_DHAR(s) + 4),read_wiznet(Sn_DHAR(s) + 5));    
	 */
}

/////Configura direccion IP destino
void wr_Sn_DIPR(SOCKET s, unsigned char * addr) {
	write_wiznet((Sn_DIPR(s) + 0), addr[0]);
	write_wiznet((Sn_DIPR(s) + 1), addr[1]);
	write_wiznet((Sn_DIPR(s) + 2), addr[2]);
	write_wiznet((Sn_DIPR(s) + 3), addr[3]);
}

///// funcion obtiene la direccion IP destino
void rd_Sn_DIPR(SOCKET s, unsigned char * addr) {
	addr[0] = read_wiznet(Sn_DIPR(s));
	addr[1] = read_wiznet(Sn_DIPR(s) + 1);
	addr[2] = read_wiznet(Sn_DIPR(s) + 2);
	addr[3] = read_wiznet(Sn_DIPR(s) + 3);
	/*printf("leyendo direccin IP destino: %d.%d.%d.%d\n\n",\
read_wiznet(Sn_DIPR(s) + 0),read_wiznet(Sn_DIPR(s) + 1),read_wiznet(Sn_DIPR(s) + 2),read_wiznet(Sn_DIPR(s) + 3));
	 */
}

////funcin para configuracion del puerto destino
void wr_Sn_DPORT(SOCKET s, unsigned char * addr) {
	write_wiznet((Sn_DPORT(s) + 0), addr[0]);
	write_wiznet((Sn_DPORT(s) + 1), addr[1]);
}

/////Lectura puerto destino
void rd_Sn_DPORT(SOCKET s, unsigned char * addr) {
	addr[0] = read_wiznet(Sn_DPORT(s));
	addr[1] = read_wiznet(Sn_DPORT(s) + 1);

	/*printf("leyendo puerto destino: %d%d\n",\
read_wiznet(Sn_DPORT(s)),read_wiznet(Sn_DPORT(s) + 1));*/
}

/**
 @brief  This sets the maximum segment size of TCP in Active Mode), while in Passive Mode this is set by peer
 */
void wr_Sn_MSS(SOCKET s, unsigned short Sn_MSSR0) {
	write_wiznet(Sn_MSSR(s), (unsigned char) ((Sn_MSSR(s) & 0xff00) >> 8));
	write_wiznet((Sn_MSSR(s) + 1), (unsigned char) (Sn_MSSR(s) & 0x00ff));
}

void wr_Sn_TTL(SOCKET s, unsigned char ttl) {
	write_wiznet(Sn_TTL(s), ttl);
}

/**
 @brief  These below function is used to setup the Protocol Field of IP Header when
 executing the IP Layer RAW mode.
 */
void wr_Sn_PROTO(SOCKET s, unsigned char proto) {
	write_wiznet(Sn_PROTO(s), proto);
}

/**
 @brief  get socket interrupt status

 These below functions are used to read the Interrupt & Soket Status register
 */
unsigned char rd_Sn_IR(SOCKET s) {
	return read_wiznet(Sn_IR(s));
}

/**
 @brief   get socket status
 */
unsigned char rd_Sn_SR(SOCKET s) {
	return read_wiznet(Sn_SR(s));
}

/**
 @brief  get socket TX free buf size

 This gives free buffer size of transmit buffer. This is the data size that user can transmit.
 User shuold check this value first and control the size of transmitting data
 */
unsigned short rd_Sn_TX_FSR(SOCKET s) {
	unsigned short val = 0, val1 = 0;
	do {
		val1 = read_wiznet(Sn_TX_FSR(s));
		val1 = (val1 << 8) + read_wiznet(Sn_TX_FSR(s) + 1);
		if (val1 != 0) {
			val = read_wiznet(Sn_TX_FSR(s));
			val = (val << 8) + read_wiznet(Sn_TX_FSR(s) + 1);
		}
	} while (val != val1);
	return val;
}

/**
 @brief   get socket RX recv buf size

 This gives size of received data in receive buffer.
 */
unsigned short rd_Sn_RX_RSR(SOCKET s) {
	unsigned short val = 0, val1 = 0;
	do {
		val1 = read_wiznet(Sn_RX_RSR(s));
		val1 = (val1 << 8) + read_wiznet(Sn_RX_RSR(s) + 1);
		if (val1 != 0) {
			val = read_wiznet(Sn_RX_RSR(s));
			val = (val << 8) + read_wiznet(Sn_RX_RSR(s) + 1);
		}
	} while (val != val1);
	return val;
}

/******************************************************************************************
 Descripcion: Funcin de inicializacin del W5100 configura los parametros de inicializacion
 del modulo tales como direccion ip hardware, gateway y mascara de red
 Ejemplo:
 red_init();

 ******************************************************************************************/

int red_init(unsigned char *hw_addr, unsigned char *ip_addr,
		unsigned char *sub_mask, unsigned char *gtw_addr) {

// configuracion del W5100 Mode Register: 0x0000
// reseteamos para limpiar la configuracion anterior y borrar memoria
	if (!reset_wiznet()) {

#if DEBUG_W5100>0
		printf("Imposible reiniciar Wiznet \n");
#endif

		return 0;
	}
	//write_wiznet(MR, RST);            // MR = 0b10000000;
	//wsleep(1000);

//Escribiendo en el Wiznet W5100 Source Hardware Address  (SHAR): 0x0001 to 0x0004
	if (!write_wiznet(SHAR + 0, hw_addr[0])
			|| !write_wiznet(SHAR + 1, hw_addr[1])
			|| !write_wiznet(SHAR + 2, hw_addr[2])
			|| !write_wiznet(SHAR + 3, hw_addr[3])
			|| !write_wiznet(SHAR + 4, hw_addr[4])
			|| !write_wiznet(SHAR + 5, hw_addr[5])) {
		return 0;
	}

//Escribiendo en el Wiznet W5100 Gateway Address   (GAR): 0x0001 to 0x0004
	if (!write_wiznet(GAR + 0, gtw_addr[0])
			|| !write_wiznet(GAR + 1, gtw_addr[1])
			|| !write_wiznet(GAR + 2, gtw_addr[2])
			|| !write_wiznet(GAR + 3, gtw_addr[3])) {
		return 0;
	}

//Escribiendo en el Wiznet W5100 Sub Mask Address (SUBR): 0x0005 to 0x0008
	if (!write_wiznet(SUBR + 0, sub_mask[0])
			|| !write_wiznet(SUBR + 1, sub_mask[1])
			|| !write_wiznet(SUBR + 2, sub_mask[2])
			|| !write_wiznet(SUBR + 3, sub_mask[3])) {
		return 0;
	}

// Escribiendo en el Wiznet W5100 IP Address (SIPR): 0x000F to 0x0012
	if (!write_wiznet(SIPR + 0, ip_addr[0])
			|| !write_wiznet(SIPR + 1, ip_addr[1])
			|| !write_wiznet(SIPR + 2, ip_addr[2])
			|| !write_wiznet(SIPR + 3, ip_addr[3])) {
		return 0;
	}

//printf("se escriben los parametros de configuracion de memoria Wiznet RMSR and TMSR\n\n");
	// sysinit(0x16,0x16);

	if (!write_wiznet(RMSR, 0x16) || !write_wiznet(TMSR, 0x16)) {
		return 0;
	}
#if DEBUG_W5100>0
	printf("read RMSR %x\n", read_wiznet(RMSR));
	printf("read TMSR %x\n", read_wiznet(TMSR));
#endif

	if (!write_wiznet(RTR, 0x0F) || !write_wiznet(RTR + 1, 0xA0)) {
		return 0;
	}

#if DEBUG_W5100>0
	printf("Wiznet W5100 Inicializado!\n%d\n",read_wiznet(RMSR));
#endif
	return 1;
} //// END INIT W5100Z

